=begin

(c) TIG 2009

  entities-add_linear_dimension.rb
  
Adds new Method the Sketchup Entities class

  add_linear_dimension(start_point, end_point, offset, rotation)
  
  The first pair are Sketchup::Point3d's (or as Arrays [x,y,z]).
  The second pair are both optional:  
  . 'offset' determines the offset from the two points.
    default=0.0, as number [==inches] or units [e.g. 50.mm or 2.inches] or a string 
    that could be read as a length [e.g. "50mm" or "2\""].
    Note that there are NO leaders drawn for an offset dimension.
  . 'rotation' turns the dimension about the length-axis (of 2 points). 
    default=0.0, it is entered in degrees.
  These two work together:
    an offset of 0 and a rotation of 0 give a 'flat' dimension spanning 
    between the two points,
    an offset of 0 and a rotation of 90 give a 'vertically' oriented 
    dimension spanning between the two points,
    an offset of 2" with a rotation of 0 gives a 'flat' dimension offset 
    from the two points,
    an offset of 2" and a rotation of 90 gives a 'vertically' oriented 
    dimension offset 2" above the two points.;
    and so on...
  
Essential:
  Needs 'linear_dimension.skp' in the Plugins folder
it could be customized: keep it 1" long x 0" high.

Tip:
  If you want a dimension that is readable from any direction then add
two on top of each other, one rotated at 0 and the other at 90.  As one 
disappears from view the other will become visible...

Version:
  1.1 20090820 First Release
  
=end
require 'sketchup.rb'
class Sketchup::Entities
  def add_linear_dimension(start_point,end_point,offset=0,rotation=0)
    ### check for skp
    if not Sketchup.find_support_files("linear_dimension.skp","Plugins")[0]
      puts "../Plugins/linear_dimension.skp NOT FOUND."; return nil
    end#if
    start_point=Geom::Point3d.new(start_point)if start_point.class==Array
    end_point=Geom::Point3d.new(end_point)if end_point.class==Array
    offset=0 if not offset
    offset=offset.to_l
    rotation=0 if not rotation
    rotation=rotation.to_f
    ###
    model=Sketchup.active_model
   model.start_operation("entities-add_linear_dimension")
    definitions=model.definitions
    definition_names=[]
    definitions.each{|d|definition_names<<d.name}
    model.import(Sketchup.find_support_files("linear_dimension.skp","Plugins")[0], false) if not definition_names.include?("linear_dimension")
    definition=nil
    definitions.each{|d|definition=d if d.name=="linear_dimension"}
    if not definition
      puts "'linear_dimension' definition NOT FOUND"; return nil
    end#if
    length=start_point.distance(end_point)### length
    if length==0
      puts "A 'linear_dimension' cannot have zero length."
      return nil
    end#if
    ### place instance
    dim_group=self.add_group()
    tp=Geom::Transformation.new(start_point)
    instance=dim_group.entities.add_instance(definition,tp)
    ### scale to length
    dim_group.transform!(Geom::Transformation.scaling(start_point,length,1.0,1.0))
    ### move to offset leader
    vecto=(end_point - start_point)
    offset_point=start_point.clone
    offset_point.offset!([0,1,0],offset)
    instance.transform!(Geom::Transformation.new(offset_point))
    ### rotate about length-axis
    dim_group.transform!(Geom::Transformation.rotation(start_point,[1,0,0],rotation.degrees))
    ### rotation angles to suit in Z
    ### adjust angles to suit 'quadrant' 
    if((end_point.y-start_point.y)>0 and(end_point.x-start_point.x)>0)or((end_point.y-start_point.y)<0 and(end_point.x-start_point.x)>0)
      anglez= Math::atan((end_point.y-start_point.y)/(end_point.x-start_point.x))
    elsif((end_point.y-start_point.y)>0 and(end_point.x-start_point.x)<0)or((end_point.y-start_point.y)<0 and(end_point.x-start_point.x)<0)
      anglez= Math::atan((end_point.y-start_point.y)/(end_point.x-start_point.x))+180.degrees
    elsif(end_point.x-start_point.x)==0 and(end_point.y-start_point.y)!=0
      if(end_point.y-start_point.y)>0
        anglez= 90.degrees
      elsif(end_point.y-start_point.y)<0
        anglez= 270.degrees
      end#if
    elsif(end_point.y-start_point.y)==0 and(end_point.x-start_point.x)!=0
      if(end_point.x-start_point.x)>0
        anglez= 0
      elsif(end_point.x-start_point.x)<0
        anglez= 180.degrees
      end#if
    elsif(end_point.x-start_point.x)==0 and(end_point.y-start_point.y)==0
      anglez= 0
    else
      anglez= 0
    end#if
    ### spin in Z
    trz=Geom::Transformation.rotation(start_point,[0,0,1],anglez)
    dim_group.transform!(trz)
    ### now rotate 'up/dn' so it's at end_point.z
    if(end_point.x-start_point.x)==0 and(end_point.y-start_point.y)==0
      ### it's 'vertical'
      if(end_point.z-start_point.z)>0
        trh=Geom::Transformation.rotation(start_point,[0,1,0],-90.degrees)
      elsif(end_point.z-start_point.z)<0
        trh=Geom::Transformation.rotation(start_point,[0,1,0],90.degrees)
      else ### z==0 too !
        puts "Zero length 'linear_dimension' not allowed."; return nil
      end#if
    else
      angle= -1*Math::asin((end_point.z - start_point.z)/(length))
      vector=Geom::Point3d.new(end_point.x,end_point.y,start_point.z) - start_point
      trr=Geom::Transformation.rotation(start_point,[0,0,1],90.degrees)
      vector.transform!(trr) ### at 90 degrees to line
      trh=Geom::Transformation.rotation(start_point,vector,angle)
    end#if
    ###
    dim_group.transform!(trh) ### spin to end_point
    ###
    instance.explode
    dim_group.explode
    ###
   model.commit_operation
    ###
  end#def
end#class